#! /usr/bin/env python3

import numpy as np

def evalRate():
    """
    Evaluates nucleogenic production rates of neutrons and of argon-39.

    Modify line
        wtfrac = 
    to select compositional estimate.

    Optionally modify values in
        wtfrac_usr = 
    if using your own compositional estimate.

    Show this help in unix shell:  pydoc ./evalProdRates.py

    Run in unix shell:  python3 evalProdRates.py

    Outputs on screen.

    Redirect output to file:  python3 evalProdRates.py > outfile

    Relates to publication 
    O. Šrámek, L. Stevens, W. F. McDonough, S. Mukhopadhyay, and 
    J. Peterson: “Subterranean production of neutrons, 39Ar and 
    21Ne: Rates and uncertainties.” Submitted to Geochim. Cosmochim. Acta
    (arXiv:1509.07436).

    Ondřej Šrámek, Charles Univ Prague, ondrej.sramek@gmail.com, Sep 2015
    """

    wtfrac_z = np.array([6, 8, 9, 11, 12, 13, 14, 19, 20, 22, 26, 90, 92])
    wtfrac_e = ("C", "O", "F", "Na", "Mg", "Al", "Si", "K", "Ca", "Ti", "Fe",
                "Th", "U")

    # upper continental crust (Rudnick & Gao 2014 TGC)
    wtfrac_ucc = np.array([1.36E-03, 0.480, 5.57E-04, 0.0243, 0.0150, 0.0815,
                0.311, 0.0232, 0.0257, 3.84E-03, 0.0385, 1.05E-05, 2.7E-06])

    # middle continental crust (Rudnick & Gao 2014 TGC)
    wtfrac_mcc = np.array([2.73E-04, 0.469, 5.24E-04, 0.0251, 0.0216, 0.0794,
                0.297, 0.0191, 0.0375, 4.14E-03, 0.0460, 6.5E-06, 1.3E-06])

    # lower continental crust (Rudnick & Gao 2014 TGC)
    wtfrac_lcc = np.array([5.46E-05, 0.452, 5.70E-04, 0.0197, 0.0437, 0.0894,
                0.250, 5.06E-03, 0.0685, 4.91E-03, 0.0655, 1.2E-06, 2E-07])

    # bulk oceanic crust (White & Klein 2014 TGC)
    wtfrac_boc = np.array([1.36E-04, 0.446, 0., 0.0164, 0.0621, 0.0831, 0.234,
                6.51E-04, 0.0843, 6.59E-03, 0.0635, 2.10E-07, 7.00E-08])

    # depleted morb mantle (Salters & Stracke 2004 G3)
    wtfrac_dmm = np.array([1.37E-05, 0.440, 1.10E-05, 2.15E-03, 0.230, 0.0227,
                0.210, 6.00E-05, 0.0250, 7.98E-04, 0.0627, 1.37E-08, 4.70E-09])
    
    ################################################################
    # OPTIONALLY MODIFY HERE - REPLACE VALUES WITH YOUR COMPOSITION
    # user suplied composition [elemental weight fraction]
    ################################################################
    wtfrac_usr = np.array([
        1.36e-3,  #6 carbon
        0.480,    #8 oxygen
        5.57e-4,  #9 fluorine
        0.0243,   #11 sodium
        0.0150,   #12 magnesium
        0.0815,   #13 aluminum
        0.311,    #14 silicon
        0.0232,   #19 potassium
        0.0257,   #20 calcium
        3.84e-3,  #22 titanium
        0.0385,   #26 iron
        1.05E-5,  #90 thorium
        2.7E-6    #92 uranium
    ]) #############################################################

    ################################################################
    # MODIFY HERE - SELECT COMPOSITION FROM ABOVE DEFINED ESTIMATES
    ################################################################
    wtfrac = wtfrac_usr
    ################################################################

    ch = ("232Th", "235U", "238U")

    tg = ("27Al", "23Na", "29Si", "30Si", "18O", "26Mg", "25Mg", "19F",
          "17O", "56Fe", "41K", "48Ti", "13C", "44Ca", "SF")

    el = np.zeros(15) #array to multiply with elem abund of target nuclies
    el[0] = wtfrac[5] #Al
    el[1] = wtfrac[3] #Na
    el[2] = wtfrac[6] #Si
    el[3] = wtfrac[6] #Si
    el[4] = wtfrac[1] #O
    el[5] = wtfrac[4] #Mg
    el[6] = wtfrac[4] #Mg
    el[7] = wtfrac[2] #F
    el[8] = wtfrac[1] #O
    el[9] = wtfrac[10] #Fe
    el[10] = wtfrac[7] #K
    el[11] = wtfrac[9] #Ti
    el[12] = wtfrac[0] #C
    el[13] = wtfrac[8] #Ca
    el[14] =  1. #SF

    pp = np.zeros(3) #array to multiply with elem abund of chain parents
    pp[0] = wtfrac[11] #Th
    pp[1] = wtfrac[12] #U
    pp[2] = wtfrac[12] #U

    cn = np.array([
        [2.65E+09, 3.31E+08, 5.03E+09],
        [6.07E+09, 8.02E+08, 1.23E+10],
        [1.95E+08, 2.53E+07, 3.91E+08],
        [1.68E+08, 2.05E+07, 3.17E+08],
        [8.77E+07, 1.33E+07, 2.27E+08],
        [1.72E+09, 2.41E+08, 3.72E+09],
        [1.01E+09, 1.44E+08, 2.22E+09],
        [1.60E+10, 2.34E+09, 3.75E+10],
        [9.50E+06, 1.43E+06, 2.47E+07],
        [1.28E+08, 3.35E+06, 9.55E+07],
        [1.10E+08, 8.92E+06, 1.64E+08],
        [4.35E+08, 2.20E+07, 5.00E+08],
        [3.61E+08, 5.48E+07, 9.96E+08],
        [2.98E+07, 2.22E+06, 4.29E+07],
        [3.01E+03, 2.37E+03, 4.44E+08]
    ])

    ca = np.array([
        [2.59E+08, 2.36E+07, 3.92E+08],
        [5.04E+08, 4.46E+07, 8.02E+08],
        [3.19E+07, 4.05E+06, 6.46E+07],
        [1.37E+07, 1.35E+06, 2.51E+07],
        [1.97E+07, 2.89E+06, 4.68E+07],
        [3.92E+08, 5.43E+07, 8.13E+08],
        [2.75E+08, 4.04E+07, 6.33E+08],
        [1.78E+09, 2.06E+08, 3.17E+09],
        [2.34E+06, 3.39E+05, 5.70E+06],
        [6.81E+06, 2.53E+04, 1.62E+06],
        [1.06E+07, 4.72E+05, 1.22E+07],
        [5.88E+07, 2.29E+06, 5.89E+07],
        [1.31E+08, 2.10E+07, 4.02E+08],
        [3.96E+06, 2.13E+05, 4.80E+06],
        [2.88E+02, 3.09E+02, 4.73E+07]
    ])

    # output composition
    print ()
    print ("Rock composition")
    for i in range(wtfrac.size):
        s = '{:2}   {:2}{:11.2e}'.format( wtfrac_z[i], wtfrac_e[i], wtfrac[i] )
        print (s)
    print ()
    
    # calculate, output neutron production rate
    nr = np.zeros(cn.shape)
    print ("Neutron production rate per year per kg-rock")
    s = '{:13}{:12}{:12}{:12}{:12}'.format('', ch[0],ch[1],ch[2], 'sum_target')
    print (s)
    for i in range(cn.shape[0]):
        for j in range(cn.shape[1]):
            nr[i,j] = cn[i,j] * el[i] * pp[j]
        s = '{:9}{:12.2e}{:12.2e}{:12.2e}{:12.2e}'.format( tg[i],
                nr[i,0], nr[i,1], nr[i,2], nr[i,:].sum() )
        print (s)
    s = '{}{:12.2e}{:12.2e}{:12.2e}'.format( 'sum_chain',
            nr[:,0].sum(), nr[:,1].sum(), nr[:,2].sum() )
    print (s)
    print ('Grand total = {:9.2e} neutrons/year/kg-rock'.format( nr.sum() ))
    print ()

    # calculate, output argon-39 production rate
    ar = np.zeros(ca.shape)
    print ("Argon-39 production rate per year per kg-rock")
    s = '{:13}{:12}{:12}{:12}{:12}'.format('', ch[0],ch[1],ch[2], 'sum_target')
    print (s)
    for i in range(ca.shape[0]):
        for j in range(ca.shape[1]):
            ar[i,j] = ca[i,j] * el[i] * pp[j] * el[10]
        s = '{:9}{:12.2e}{:12.2e}{:12.2e}{:12.2e}'.format( tg[i],
                ar[i,0], ar[i,1], ar[i,2], ar[i,:].sum() )
        print (s)
    s = '{}{:12.2e}{:12.2e}{:12.2e}'.format( 'sum_chain',
            ar[:,0].sum(), ar[:,1].sum(), ar[:,2].sum() )
    print (s)
    print ('Grand total = {:9.2e} atoms/year/kg-rock'.format( ar.sum() ))
    print ()
    
    return()


if __name__ == "__main__":
    """
    To do if the script is executed at command line.
    """
    evalRate()
